module types   

   real, parameter :: epsilon=.0001

   type stack_3d
      integer                :: max, index
      real, dimension(100,3) :: coord
   end type stack_3d  

   type stack_1d
      integer                :: max, index
      integer,dimension(100) :: content
   end type stack_1d  

end
module constants
 double precision :: pi
 parameter(pi=3.14159)
end

module elastic 
  character(20)  :: title='',outputfile=''
  double precision, dimension(6,6) :: C=0,S=0,CP=0
  integer :: n_phi=0, n_theta=0
  logical :: verbose=.FALSE., do_young=.FALSE., do_sound=.FALSE., do_compress=.FALSE., do_poisson=.FALSE.
end 

module vrml3d
  use types
  use elastic
  use constants
  integer, dimension(10300,4) :: faces=0
  integer :: n_phif=0, n_thetaf=0, n_faces
  
contains
  subroutine vrml3d_header
    write(4,'(A)') '#VRML V2.0 utf8'
    write(4,*) ''
    write(4,*) 'Viewpoint {'
    write(4,*) '  fieldofview 0.78539'
    write(4,*) '  position 0 0 1'
    write(4,*) '  orientation 0 0 1 0'
    write(4,*) '  jump TRUE'
    write(4,*) '  description "v1"'
    write(4,*) '}'  
    write(4,*) 'Viewpoint {'
    write(4,*) '  fieldofview 0.78539'
    write(4,*) '  position 0 0 10'
    write(4,*) '  orientation 0 0 1 0'
    write(4,*) '  jump TRUE'
    write(4,*) '  description "v10"'
    write(4,*) '}'  
    write(4,*) 'Viewpoint {'
    write(4,*) '  fieldofview 0.78539'
    write(4,*) '  position 0 0 100'
    write(4,*) '  orientation 0 0 1 0'
    write(4,*) '  jump TRUE'
    write(4,*) '  description "v100"'
    write(4,*) '}'  
    write(4,*) 'Viewpoint {'
    write(4,*) '  fieldofview 0.78539'
    write(4,*) '  position 0 0 1000'
    write(4,*) '  orientation 0 0 1 0'
    write(4,*) '  jump TRUE'
    write(4,*) '  description "v1000"'
    write(4,*) '}'  


    
  end subroutine vrml3d_header
  
  subroutine vrml3d_shape_open
    write(4,*) 'Shape {'
    write(4,*) '  geometry IndexedFaceSet {'
  end subroutine vrml3d_shape_open
  
  subroutine vrml3d_shape_close
    write(4,*) '}'
  end subroutine vrml3d_shape_close
  
  subroutine vrml3d_shape_appearance(r,g,b,t)
    real :: r,g,b,t
    write(4,*) '    appearance Appearance {'
    write(4,*) '       material Material {'
    write(4,*) '         diffuseColor',r,g,b
    write(4,*) '         transparency',t
    write(4,*) '       }'
    write(4,*) '     }'     
  end subroutine vrml3d_shape_appearance

  subroutine vrml3d_end
  end subroutine vrml3d_end


  
  subroutine vrml3d_spheroid
    integer :: j,kk, index
    index=0
    write(4,*) '    coordIndex ['
    do j=0, n_phif
      write(4,'(4(I5A1))',ADVANCE='NO') (faces(index,kk),',', kk=1,3), -1,','
      index=index+1
    enddo
    write(4,*)
    do i=2,n_thetaf-1
      do j=0, n_phif
        write(4,'(5(I5A1))',ADVANCE='NO') (faces(index,kk),',', kk=1,4), -1,','
        index=index+1        
      enddo
      write(4,*)
    enddo
    do j=0, n_phif-1
      write(4,'(4(I5A1))',ADVANCE='NO') (faces(index,kk),',', kk=1,3), -1,','
      index=index+1
    enddo             
    j=n_phif
    write(4,'(3(I5A1))',ADVANCE='NO') (faces(index,kk),',', kk=1,3)
    write(4,'(I5)') -1
    write(4,*) "]"
 
  end subroutine vrml3d_spheroid



  subroutine vrml3d_color_sign(points,r1,g1,b1,r2,g2,b2)
    integer :: i,j, index, n_phif2
    double precision, dimension(10300) :: points
    double precision, dimension(3) :: k=0
    double precision :: theta, phi
    real :: r,g,b,r1,g1,b1,r2,g2,b2


    write(4,*) '  color Color {'
    write(4,*) '      color [ '
         
    index=0
    do i=0,n_thetaf-1
      theta=dble(i)/dble(n_thetaf)*pi
      n_phif2=n_phif;if (i==0 .OR. i==n_thetaf) n_phif2=0  

      do j=0, n_phif2
         if (points(index).LE.0) then
           r=r2;g=g2;b=b2;
         else
           r=r1;g=g1;b=b1;
         endif    

         index=index+1
         write(4,'(3F5.1,A1)',ADVANCE='NO') r,g,b,','
!    write(4,*) i,j, theta, phi, n_thetaf, n_phif

      enddo
    enddo
    i=n_thetaf
    j=0
    if (points(index).LE.0) then
      r=r2;g=g2;b=b2;
    else
      r=r1;g=g1;b=b1;
    endif    
    write(4,'(3F5.1)') r,g,b
    write(4,*) '      ]'
    write(4,*) '    }'

          
  end subroutine vrml3d_color_sign

  
  subroutine vrml3d_points(points)
    integer :: i,j, index, n_phif2
    double precision, dimension(10300) :: points
    double precision, dimension(3) :: k=0
    double precision :: theta, phi


    write(4,*) '  coord Coordinate {'
    write(4,*) '      point [ '
         
    index=0
    do i=0,n_thetaf-1
      theta=dble(i)/dble(n_thetaf)*pi
      n_phif2=n_phif;if (i==0 .OR. i==n_thetaf) n_phif2=0  

      do j=0, n_phif2
         phi=dble(j)/dble(n_phif)*2d0*pi
         k(1)=sin(theta)*cos(phi)*abs(points(index))
         k(2)=sin(theta)*sin(phi)*abs(points(index))
         k(3)=cos(theta)*abs(points(index))
         index=index+1
         write(4,'(3F10.3,A1)',ADVANCE='NO') k(1),k(2),k(3),','
!    write(4,*) i,j, theta, phi, n_thetaf, n_phif

      enddo
    enddo
    i=n_thetaf
    j=0
    theta=dble(i)/dble(n_thetaf)*pi
    phi=dble(j)/dble(n_phif)*2d0*pi
    k(1)=sin(theta)*cos(phi)*abs(points(index))
    k(2)=sin(theta)*sin(phi)*abs(points(index))
    k(3)=cos(theta)*abs(points(index))
    write(4,'(3F10.3)') k(1),k(2),k(3)
    write(4,*) '      ]'
    write(4,*) '    }'
    write(4,*) '  }'
          
  end subroutine vrml3d_points


  

  subroutine vrml3d_calculate_spheroid
    integer :: i,j, index

!   if (n_phi.LE.100) then
!     n_phif=n_phi
!   else
!     n_phif=10
!   endif
!   if (n_theta.LE.100) then
!     n_thetaf=n_theta
!   else
!     n_thetaf=10
!   endif
        
    index=0
    do j=0, n_phif-1
      faces(index,1)=0
      faces(index,2)=j+1
      faces(index,3)=j+2
!      write (*,*) faces(index,1),faces(index,2),faces(index,3)
      index=index+1
    enddo
      faces(index,1)=0
      faces(index,2)=n_phif+1
      faces(index,3)=1
      index=index+1
    do i=2,n_thetaf-1
      do j=0, n_phif-1
        faces(index,1)=(i-2)*(n_phif+1)+j+1
        faces(index,2)=(i-1)*(n_phif+1)+j+1
        faces(index,3)=(i-1)*(n_phif+1)+j+2
        faces(index,4)=(i-2)*(n_phif+1)+j+2
        index=index+1
      enddo
      faces(index,1)=(i-1)*(n_phif+1)
      faces(index,2)=i*(n_phif+1)
      faces(index,3)=(i-1)*(n_phif+1)+1
      faces(index,4)=(i-2)*(n_phif+1)+1
      index=index+1
    enddo
    do j=0, n_phif-1
      faces(index,1)=(n_thetaf-2)*(n_phif+1)+j+1
      faces(index,2)=(n_thetaf-1)*(n_phif+1)+1
      faces(index,3)=(n_thetaf-2)*(n_phif+1)+j+2
      index=index+1
    enddo
    faces(index,1)=(n_thetaf-1)*(n_phif+1)
    faces(index,2)=(n_thetaf-1)*(n_phif+1)+1
    faces(index,3)=(n_thetaf-2)*(n_phif+1)+1
    n_faces=index 
  end subroutine vrml3d_calculate_spheroid
end module vrml3d



